@@ -1,7 +1,7 @@
 //! Methods for expanding structs
 use std::collections::{HashMap, VecDeque};
 
-use anyhow::{Context as _, Result};
+use eyre::{eyre, Result};
 use inflector::Inflector;
 use proc_macro2::TokenStream;
 use quote::quote;
@@ -55,17 +55,18 @@ impl Context {
 
     /// Generates the type definition for the name that matches the given identifier
     fn generate_internal_struct(&self, id: &str) -> Result<TokenStream> {
-        let sol_struct = self.internal_structs.structs.get(id).context("struct not found")?;
+        let sol_struct =
+            self.internal_structs.structs.get(id).ok_or_else(|| eyre!("struct not found"))?;
         let struct_name = self
             .internal_structs
             .rust_type_names
             .get(id)
-            .context(format!("No types found for {}", id))?;
+            .ok_or_else(|| eyre!("No types found for {}", id))?;
         let tuple = self
             .internal_structs
             .struct_tuples
             .get(id)
-            .context(format!("No types found for {}", id))?
+            .ok_or_else(|| eyre!("No types found for {}", id))?
             .clone();
         self.expand_internal_struct(struct_name, sol_struct, tuple)
     }
@@ -90,27 +91,43 @@ impl Context {
         tuple: ParamType,
     ) -> Result<TokenStream> {
         let mut fields = Vec::with_capacity(sol_struct.fields().len());
+
+        // determines whether we have enough info to create named fields
+        let is_tuple = sol_struct.has_nameless_field();
+
         for field in sol_struct.fields() {
-            let field_name = util::ident(&field.name().to_snake_case());
-            match field.r#type() {
-                FieldType::Elementary(ty) => {
-                    let ty = types::expand(ty)?;
-                    fields.push(quote! { pub #field_name: #ty });
-                }
-                FieldType::Struct(struct_ty) => {
-                    let ty = expand_struct_type(struct_ty);
-                    fields.push(quote! { pub #field_name: #ty });
-                }
+            let ty = match field.r#type() {
+                FieldType::Elementary(ty) => types::expand(ty)?,
+                FieldType::Struct(struct_ty) => expand_struct_type(struct_ty),
                 FieldType::Mapping(_) => {
-                    return Err(anyhow::anyhow!(
-                        "Mapping types in struct `{}` are not supported {:?}",
-                        name,
-                        field
-                    ))
+                    eyre::bail!("Mapping types in struct `{}` are not supported {:?}", name, field)
                 }
+            };
+
+            if is_tuple {
+                fields.push(ty);
+            } else {
+                let field_name = util::safe_ident(&field.name().to_snake_case());
+                fields.push(quote! { pub #field_name: #ty });
             }
         }
 
+        let name = util::ident(name);
+
+        let struct_def = if is_tuple {
+            quote! {
+                pub struct #name(
+                    #( #fields ),*
+                );
+            }
+        } else {
+            quote! {
+                pub struct #name {
+                    #( #fields ),*
+                }
+            }
+        };
+
         let sig = if let ParamType::Tuple(ref tokens) = tuple {
             tokens.iter().map(|kind| kind.to_string()).collect::<Vec<_>>().join(",")
         } else {
@@ -121,27 +138,24 @@ impl Context {
 
         let abi_signature_doc = util::expand_doc(&format!("`{}`", abi_signature));
 
-        let name = util::ident(name);
-
         // use the same derives as for events
         let derives = util::expand_derives(&self.event_derives);
 
         let ethers_contract = ethers_contract_crate();
         Ok(quote! {
             #abi_signature_doc
-            #[derive(Clone, Debug, Default, Eq, PartialEq, #ethers_contract::EthAbiType, #derives)]
-            pub struct #name {
-                #( #fields ),*
-            }
+            #[derive(Clone, Debug, Default, Eq, PartialEq, #ethers_contract::EthAbiType, #ethers_contract::EthAbiCodec, #derives)]
+            #struct_def
         })
     }
 
     fn generate_human_readable_struct(&self, name: &str) -> Result<TokenStream> {
-        let sol_struct = self.abi_parser.structs.get(name).context("struct not found")?;
+        let sol_struct =
+            self.abi_parser.structs.get(name).ok_or_else(|| eyre!("struct not found"))?;
         let mut fields = Vec::with_capacity(sol_struct.fields().len());
         let mut param_types = Vec::with_capacity(sol_struct.fields().len());
         for field in sol_struct.fields() {
-            let field_name = util::ident(&field.name().to_snake_case());
+            let field_name = util::safe_ident(&field.name().to_snake_case());
             match field.r#type() {
                 FieldType::Elementary(ty) => {
                     param_types.push(ty.clone());
@@ -157,18 +171,14 @@ impl Context {
                         .abi_parser
                         .struct_tuples
                         .get(name)
-                        .context(format!("No types found for {}", name))?
+                        .ok_or_else(|| eyre!("No types found for {}", name))?
                         .clone();
                     let tuple = ParamType::Tuple(tuple);
 
                     param_types.push(struct_ty.as_param(tuple));
                 }
                 FieldType::Mapping(_) => {
-                    return Err(anyhow::anyhow!(
-                        "Mapping types in struct `{}` are not supported {:?}",
-                        name,
-                        field
-                    ))
+                    eyre::bail!("Mapping types in struct `{}` are not supported {:?}", name, field)
                 }
             }
         }
@@ -191,7 +201,7 @@ impl Context {
 
         Ok(quote! {
             #abi_signature_doc
-            #[derive(Clone, Debug, Default, Eq, PartialEq, #ethers_contract::EthAbiType, #derives)]
+            #[derive(Clone, Debug, Default, Eq, PartialEq, #ethers_contract::EthAbiType, #ethers_contract::EthAbiCodec, #derives)]
             pub struct #name {
                 #( #fields ),*
             }
@@ -312,6 +322,22 @@ impl InternalStructs {
             .map(String::as_str)
     }
 
+    /// Returns the name of the rust type that will be generated if the given output is a struct
+    /// NOTE: this does not account for arrays or fixed arrays
+    pub fn get_function_output_struct_type(
+        &self,
+        function: &str,
+        internal_type: &str,
+    ) -> Option<&str> {
+        self.outputs
+            .get(function)
+            .and_then(|outputs| {
+                outputs.iter().find(|s| s.as_str() == struct_type_identifier(internal_type))
+            })
+            .and_then(|id| self.rust_type_names.get(id))
+            .map(String::as_str)
+    }
+
     /// Returns the mapping table of abi `internal type identifier -> rust type`
     pub fn rust_type_names(&self) -> &HashMap<String, String> {
         &self.rust_type_names
