@@ -1,4 +1,5 @@
 #![deny(missing_docs, unsafe_code)]
+#![deny(rustdoc::broken_intra_doc_links)]
 
 //! Module for generating type-safe bindings to Ethereum smart contracts. This
 //! module is intended to be used either indirectly with the `abigen` procedural
@@ -19,11 +20,16 @@ mod rustfmt;
 mod source;
 mod util;
 
+pub mod multi;
+pub use multi::MultiAbigen;
+
 pub use ethers_core::types::Address;
 pub use source::Source;
 pub use util::parse_address;
 
-use anyhow::Result;
+use crate::contract::ExpandedContract;
+use eyre::Result;
+use inflector::Inflector;
 use proc_macro2::TokenStream;
 use std::{collections::HashMap, fs::File, io::Write, path::Path};
 
@@ -33,10 +39,13 @@ use std::{collections::HashMap, fs::File, io::Write, path::Path};
 /// [still not supported by Vyper](https://github.com/vyperlang/vyper/issues/1931), so you must adjust your ABIs and replace
 /// `constant` functions with `view` or `pure`.
 ///
+/// To generate bindings for _multiple_ contracts at once see also [`crate::MultiAbigen`].
+///
 /// # Example
 ///
-/// Running the command below will generate a file called `token.rs` containing the
-/// bindings inside, which exports an `ERC20Token` struct, along with all its events.
+/// Running the code below will generate a file called `token.rs` containing the
+/// bindings inside, which exports an `ERC20Token` struct, along with all its events. Put into a
+/// `build.rs` file this will generate the bindings during `cargo build`.
 ///
 /// ```no_run
 /// # use ethers_contract_abigen::Abigen;
@@ -44,6 +53,7 @@ use std::{collections::HashMap, fs::File, io::Write, path::Path};
 /// Abigen::new("ERC20Token", "./abi.json")?.generate()?.write_to_file("token.rs")?;
 /// # Ok(())
 /// # }
+#[derive(Debug, Clone)]
 pub struct Abigen {
     /// The source of the ABI JSON for the contract whose bindings
     /// are being generated.
@@ -79,8 +89,22 @@ impl Abigen {
         })
     }
 
+    /// Attempts to load a new builder from an ABI JSON file at the specific
+    /// path.
+    pub fn from_file(path: impl AsRef<Path>) -> Result<Self> {
+        let name = path
+            .as_ref()
+            .file_stem()
+            .ok_or_else(|| eyre::format_err!("Missing file stem in path"))?
+            .to_str()
+            .ok_or_else(|| eyre::format_err!("Unable to convert file stem to string"))?;
+
+        Self::new(name, std::fs::read_to_string(path.as_ref())?)
+    }
+
     /// Manually adds a solidity event alias to specify what the event struct
     /// and function name will be in Rust.
+    #[must_use]
     pub fn add_event_alias<S1, S2>(mut self, signature: S1, alias: S2) -> Self
     where
         S1: Into<String>,
@@ -93,6 +117,7 @@ impl Abigen {
     /// Manually adds a solidity method alias to specify what the method name
     /// will be in Rust. For solidity methods without an alias, the snake cased
     /// method name will be used.
+    #[must_use]
     pub fn add_method_alias<S1, S2>(mut self, signature: S1, alias: S2) -> Self
     where
         S1: Into<String>,
@@ -107,6 +132,7 @@ impl Abigen {
     ///
     /// Note that in case `rustfmt` does not exist or produces an error, the
     /// unformatted code will be used.
+    #[must_use]
     pub fn rustfmt(mut self, rustfmt: bool) -> Self {
         self.rustfmt = rustfmt;
         self
@@ -116,6 +142,7 @@ impl Abigen {
     ///
     /// This makes it possible to for example derive serde::Serialize and
     /// serde::Deserialize for events.
+    #[must_use]
     pub fn add_event_derive<S>(mut self, derive: S) -> Self
     where
         S: Into<String>,
@@ -127,8 +154,16 @@ impl Abigen {
     /// Generates the contract bindings.
     pub fn generate(self) -> Result<ContractBindings> {
         let rustfmt = self.rustfmt;
-        let tokens = Context::from_abigen(self)?.expand()?.into_tokens();
-        Ok(ContractBindings { tokens, rustfmt })
+        let name = self.contract_name.clone();
+        let (expanded, _) = self.expand()?;
+        Ok(ContractBindings { tokens: expanded.into_tokens(), rustfmt, name })
+    }
+
+    /// Expands the `Abigen` and returns the [`ExpandedContract`] that holds all tokens and the
+    /// [`Context`] that holds the state used during expansion.
+    pub fn expand(self) -> Result<(ExpandedContract, Context)> {
+        let ctx = Context::from_abigen(self)?;
+        Ok((ctx.expand()?, ctx))
     }
 }
 
@@ -139,6 +174,8 @@ pub struct ContractBindings {
     tokens: TokenStream,
     /// The output options used for serialization.
     rustfmt: bool,
+    /// The contract name
+    name: String,
 }
 
 impl ContractBindings {
@@ -161,6 +198,13 @@ impl ContractBindings {
         Ok(())
     }
 
+    /// Writes the bindings to a new Vec. Panics if unable to allocate
+    pub fn to_vec(&self) -> Vec<u8> {
+        let mut bindings = vec![];
+        self.write(&mut bindings).expect("allocations don't fail");
+        bindings
+    }
+
     /// Writes the bindings to the specified file.
     pub fn write_to_file<P>(&self, path: P) -> Result<()>
     where
@@ -170,9 +214,85 @@ impl ContractBindings {
         self.write(file)
     }
 
+    /// Writes the bindings to a `contract_name.rs` file in the specified
+    /// directory. The filename is the snake_case transformation of the contract
+    /// name.
+    pub fn write_module_in_dir<P>(&self, dir: P) -> Result<()>
+    where
+        P: AsRef<Path>,
+    {
+        let file = dir.as_ref().join(self.module_filename());
+        self.write_to_file(file)
+    }
+
     /// Converts the bindings into its underlying token stream. This allows it
     /// to be used within a procedural macro.
     pub fn into_tokens(self) -> TokenStream {
         self.tokens
     }
+
+    /// Generate the default module name (snake case of the contract name)
+    pub fn module_name(&self) -> String {
+        self.name.to_snake_case()
+    }
+
+    /// Generate the default filename of the module
+    pub fn module_filename(&self) -> String {
+        let mut name = self.module_name();
+        name.extend([".rs"]);
+        name
+    }
+}
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+    use ethers_solc::project_util::TempProject;
+
+    #[test]
+    fn can_generate_structs() {
+        let greeter = include_str!("../../tests/solidity-contracts/greeter_with_struct.json");
+        let abigen = Abigen::new("Greeter", greeter).unwrap();
+        let gen = abigen.generate().unwrap();
+        let out = gen.tokens.to_string();
+        assert!(out.contains("pub struct Stuff"));
+    }
+
+    #[test]
+    fn can_compile_and_generate() {
+        let tmp = TempProject::dapptools().unwrap();
+
+        tmp.add_source(
+            "Greeter",
+            r#"
+// SPDX-License-Identifier: MIT
+pragma solidity >=0.8.0;
+
+contract Greeter {
+
+    struct Inner {
+        bool a;
+    }
+
+    struct Stuff {
+        Inner inner;
+    }
+
+    function greet(Stuff calldata stuff) public view returns (Stuff memory) {
+        return stuff;
+    }
+}
+"#,
+        )
+        .unwrap();
+
+        let _ = tmp.compile().unwrap();
+
+        let abigen =
+            Abigen::from_file(tmp.artifacts_path().join("Greeter.sol/Greeter.json")).unwrap();
+        let gen = abigen.generate().unwrap();
+        let out = gen.tokens.to_string();
+        assert!(out.contains("pub struct Stuff"));
+        assert!(out.contains("pub struct Inner"));
+    }
 }
