@@ -1,6 +1,6 @@
 //! This module implements basic `rustfmt` code formatting.
 
-use anyhow::{anyhow, Result};
+use eyre::{eyre, Result};
 use std::{
     io::Write,
     process::{Command, Stdio},
@@ -18,18 +18,18 @@ where
         let stdin = rustfmt
             .stdin
             .as_mut()
-            .ok_or_else(|| anyhow!("stdin was not created for `rustfmt` child process"))?;
+            .ok_or_else(|| eyre!("stdin was not created for `rustfmt` child process"))?;
         stdin.write_all(source.as_ref().as_bytes())?;
     }
 
     let output = rustfmt.wait_with_output()?;
-    if !output.status.success() {
-        return Err(anyhow!(
-            "`rustfmt` exited with code {}:\n{}",
-            output.status,
-            String::from_utf8_lossy(&output.stderr),
-        ))
-    }
+
+    eyre::ensure!(
+        output.status.success(),
+        "`rustfmt` exited with code {}:\n{}",
+        output.status,
+        String::from_utf8_lossy(&output.stderr),
+    );
 
     let stdout = String::from_utf8(output.stdout)?;
     Ok(stdout)
